#!/usr/bin/env python

import os
import sys
import optparse
import numpy as np
#import StringIO

#global variables
personLogFile = ""
personIDLogFile = ""
MAX_STEPS = 3600
STAGE_CYCLES = 5 #assumes that a stage is at least 5 steps long, so, capture only after a stage cycle

# we need to import some python modules from the $SUMO_HOME/tools directory
if 'SUMO_HOME' in os.environ:
    tools = os.path.join(os.environ['SUMO_HOME'], 'tools')
    sys.path.append(tools)    
else:
    sys.exit("please declare environment variable 'SUMO_HOME'")


from sumolib import checkBinary  # Checks for the binary in environ vars
import traci


def get_options():
    opt_parser = optparse.OptionParser()
    opt_parser.add_option("--nogui", action="store_true",
                         default=False, help="run the commandline version of sumo")
    options, args = opt_parser.parse_args()
    return options

def openLogFiles():
    global personLogFile, personIDLogFile
    personLogFile = open("person-logs.txt", "w")
    personIDLogFile = open("person-id-logs.txt", "w")
# contains TraCI control loop
def run():
    step = 0
    personFound = 0    
    global personLogFile, MAX_STEPS, STAGE_CYCLES, personIDLogFile
    
    while traci.simulation.getMinExpectedNumber() > 0: #simulate until all vehicles arrive
        traci.simulationStep() #capture the steps of simulation 
        #print(step)        
        step = step + 1
        if step % STAGE_CYCLES != 0:
            continue
        
        strLog = ""
               
        peopleIDs = traci.person.getIDList() #Returns a list of person IDs       
        
        for i in range(len(peopleIDs)):
                #print("PersonID:",peopleIDs[i],"stage:",traci.person.getStage(peopleIDs[i]))
            #if len(traci.person.getVehicle(peopleIDs[i])) != 0: # if not empty
            currStage = traci.person.getStage(peopleIDs[i])
            
            if currStage.type == 3: #record only if driving or riding 
                
                personIDLogFile.write("PersonID: "+str(peopleIDs[i])+" StageType: "+ str(currStage.type) +"\n")
                
                try:
                    prevStage = traci.person.getStage(peopleIDs[i],-1) 
                except FatalTraCIError:
                    print("Previous Stage Error for Person",peopleIDs[i])
                    continue
                except:
                    print("Something went wrong")
                    continue
                
                vehicleID = traci.person.getVehicle(peopleIDs[i])
                
                if prevStage == None or currStage == None or vehicleID == None or vehicleID == "":
                    continue #do not record it
                strLog = strLog + "PersonID: %s" % (peopleIDs[i])
                strLog = strLog + "\t SourceStop: %s" % (prevStage.destStop)
                strLog = strLog + "\t DestStop: %s" % (currStage.destStop)
                strLog = strLog + "\t LineNo: %s" % (currStage.line)
                strLog = strLog + "\t VehicleNo: %s" % (vehicleID) + "\n"
                              
#                strLog = strLog + "PersonID: %s, current stage: %s, vehicle: %s, previous stage: %s" % (peopleIDs[i],traci.person.getStage(peopleIDs[i]),traci.person.getVehicle(peopleIDs[i]),) + "\n"
                if i % 100 == 0: #flush the output if more people are there
                    personLogFile.write(strLog)
                    strLog = ""
                    
                    
        
        if (step % 500) == 0 and len(peopleIDs) > 0:
            print("\nAt step:",step,"total public count:",len(peopleIDs))
            print("First ten people IDs",peopleIDs[0:10])
            print("PersonID:",peopleIDs[0],"stage:",traci.person.getStage(peopleIDs[0]),"vehicle:",traci.person.getVehicle(peopleIDs[0]))            
            #print("Stage Type:",type(traci.person.getStage(peopleIDs[0])))
            print("PersonID:",peopleIDs[0],"previous stage:",traci.person.getStage(peopleIDs[0],-1))
            print("Stage type:",traci.person.getStage(peopleIDs[0]).type)
            print("Stage type data-type:",type(traci.person.getStage(peopleIDs[0]).type))
            print("Total people journey captured in this session",personFound)
            personFound = 0
            #print("Total people journey failed to be captured in this session",failedTracking)
 
        if strLog != "":
            personLogFile.write(strLog)
        
        if step > MAX_STEPS:
            break
    
    # while loop has ended        
    print("total steps:",step)    
    traci.close()   #end simulation
    sys.stdout.flush()

def closeLogFiles():
    global personLogFile, personIDLogFile
    personLogFile.close()
    personIDLogFile.close()

# main entry point
if __name__ == "__main__":
    options = get_options()
    
    openLogFiles()
    
    # check binary
    if options.nogui:
        sumoBinary = checkBinary('sumo')
    else:
        sumoBinary = checkBinary('sumo-gui')

    traci.start([sumoBinary, "-c", "run.sumo.cfg"])
    run()
    
    closeLogFiles()
